const plantGenerationsData = [
  {
    "name": "Asia",
    "parent": "null",
    "fill": "black",
    "children": [
      {
        "name": "India",
        "parent": "Asia",
        "fill": "blue",
        "children": [
          {
            "name": "Delhi",
            "parent": "India",
            "fill": "red",
            "children": [
              {
                "name": "Delhi 'Rajdhani'",
                "parent": "Delhi",
                "fill": "green"
              },
              {
                "name": "Delhi 'Chandni Chowk'",
                "parent": "Delhi",
                "fill": "green"
              }
            ]
          },
          {
            "name": "Mumbai",
            "parent": "India",
            "fill": "red",
            "children": [
              {
                "name": "Mumbai 'Gateway of India'",
                "parent": "Mumbai",
                "fill": "green"
              },
              {
                "name": "Mumbai 'Marine Drive'",
                "parent": "Mumbai",
                "fill": "green"
              }
            ]
          }
        ]
      },
      {
        "name": "China",
        "parent": "Asia",
        "fill": "blue",
        "children": [
          {
            "name": "Beijing",
            "parent": "China",
            "fill": "red",
            "children": [
              {
                "name": "Beijing 'Forbidden City'",
                "parent": "Beijing",
                "fill": "green"
              },
              {
                "name": "Beijing 'Great Wall of China'",
                "parent": "Beijing",
                "fill": "green"
              }
            ]
          },
          {
            "name": "Shanghai",
            "parent": "China",
            "fill": "red",
            "children": [
              {
                "name": "Shanghai 'The Bund'",
                "parent": "Shanghai",
                "fill": "green"
              },
              {
                "name": "Shanghai 'Yu Garden'",
                "parent": "Shanghai",
                "fill": "green"
              }
            ]
          }
        ]
      }
    ]
  },
  {
    "name": "Europe",
    "parent": "null",
    "fill": "black",
    "children": [
      {
        "name": "Germany",
        "parent": "Europe",
        "fill": "blue",
        "children": [
          {
            "name": "Berlin",
            "parent": "Germany",
            "fill": "red",
            "children": [
              {
                "name": "Berlin 'Brandenburg Gate'",
                "parent": "Berlin",
                "fill": "green"
              },
              {
                "name": "Berlin 'Reichstag Building'",
                "parent": "Berlin",
                "fill": "green"
              }
            ]
          },
          {
            "name": "Munich",
            "parent": "Germany",
            "fill": "red",
            "children": [
              {
                "name": "Munich 'Marienplatz'",
                "parent": "Munich",
                "fill": "green"
              },
              {
                "name": "Munich 'Nymphenburg Palace'",
                "parent": "Munich",
                "fill": "green"
              }
            ]
          }
        ]
      },
      {
        "name": "France",
        "parent": "Europe",
        "fill": "blue",
        "children": [
          {
            "name": "Paris",
            "parent": "France",
            "fill": "red",
            "children": [
              {
                "name": "Paris 'Eiffel Tower'",
                "parent": "Paris",
                "fill": "green"
              },
              {
                "name": "Paris 'Louvre Museum'",
                "parent": "Paris",
                "fill": "green"
              }
            ]
          },
          {
            "name": "Marseille",
            "parent": "France",
            "fill": "red",
            "children": [
              {
                "name": "Marseille 'Old Port'",
                "parent": "Marseille",
                "fill": "green"
              },
              {
                "name": "Marseille 'Notre-Dame de la Garde'",
                "parent": "Marseille",
                "fill": "green"
              }
            ]
          }
        ]
      }
    ]
  }
];
// Define the margin, width and height of the visulization.
let margin = {top: 15, right: 120, bottom: 15, left: 120};
let	width = 960 - margin.right - margin.left;
let	height = 400 - margin.top - margin.bottom;
	
var i = 0, duration = 1000, root;

// Variable which holds the whole tree layout
var tree = d3.layout.tree()
	.size([height, width]);

// creates a diagonal generator that connects two points in a tree with custom projection.
var diagonal = d3.svg.diagonal()
	.projection(function(d) { return [d.y, d.x]; });

// Selecting the SVG element as the base to host the tree graph
var svg = d3.select("body").append("svg")
	.attr("width", width + margin.right + margin.left)
	.attr("height", height + margin.top + margin.bottom)
  .append("g")
	.attr("transform", "translate(" + margin.left + "," + margin.top + ")");

// assigning the zeroth element as the root of the Tree and setting the x and y 
root = plantGenerationsData[0];
root.x0 = height / 2;
root.y0 = 0;
  
// Triggering the update method which generates the further tree from root.
update(root);

// Sets the height of the SVG that hosts the Tree structure
d3.select(self.frameElement).style("height", "500px");

function update(source) {

  // Computing the descendant nodes 
  let nodes = tree.nodes(root);
  // Computing the descendant linksg
  let links = tree.links(nodes);

  // Normalize for fixed-depth.
  nodes.forEach(function(d) { d.y = d.depth * 180; });

  // Update the nodes according to the identifiers i.e., the plant generations 
  var node = svg.selectAll("g.node")
	  .data(nodes, function(d) { return d.id || (d.id = ++i); });

  // The below code introduces new nodes at the parent's previous position.
  var nodeEnter = node.enter().append("g")
	  .attr("class", "node")
	  .attr("transform", function(d) { return "translate(" + source.y0 + "," + source.x0 + ")"; });
	
  // The below code creates a circle that runs around the node
  nodeEnter.append("circle")
	  .attr("r", 1e-6)
	  .style("fill", function(d) { return d._children ? "pink" : "#fff"; });

  // The below code appends the text to each and every node.
  nodeEnter.append("text")
	  .attr("x", function(d) { return d.children || d._children ? -13 : 13; })
	  .attr("dy", ".35em")
	  .attr("text-anchor", function(d) { return d.children || d._children ? "end" : "start"; })
	  .text(function(d) { return d.name; })
	  .style("fill-opacity", 1e-6);

  // The below code apply transition to the nodes to their new position with in duration of 1000 ms
  var nodeUpdate = node.transition()
	  .duration(duration)
	  .attr("transform", function(d) { return "translate(" + d.y + "," + d.x + ")"; });

  // The below code colors the each node that is set in dataset
  nodeUpdate.select("circle")
    .attr("r", 11)
    .style("fill", function(d) {
        // Change the color based on the "fill" property of each node
        if (d.fill === "black") {
            return "green"; // Reverse the color mapping for "black"
        } else if (d.fill === "blue") {
            return "red"; // Reverse the color mapping for "blue"
        } else if (d.fill === "red") {
            return "blue"; // Reverse the color mapping for "red"
        } else if (d.fill === "green") {
            return "black"; // Reverse the color mapping for "green"
        } else {
            return "black"; // Default color if the fill property is not one of the expected values
        }
    });

  // The below code makes the plant generation name to be fully visible.
  nodeUpdate.select("text")
	  .style("fill-opacity", 1);

  // The below code updates the links between the nodes
  var link = svg.selectAll("path.link")
	  .data(links, function(d) { return d.target.id; });

  // The below code enters new links that connects the current position and the parent's previous position.
  link.enter().insert("path", "g")
	  .attr("class", "link")
	  .attr("d", function(d) {
		var o = {x: source.x0, y: source.y0};
		return diagonal({source: o, target: o});
	  });

  // Transition links to their new position.
  link.transition()
	  .duration(duration)
	  .attr("d", diagonal);
}
